/*
 * Copyright Contributors to the fusa-gcc-plugin project
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */
#include "process.hpp"

#include "gcc-plugin.h"
#include "diagnostic-core.h"
#include "pretty-print.h"

using namespace std;

namespace process {
static vector<string> split_lines(const string& str) {
    vector<std::string> result;
    string::size_type start = 0;
    string::size_type end = 0;

    while ((end = str.find('\n', start)) != string::npos) {
        result.push_back(str.substr(start, end - start));
        start = end + 1;
    }

    result.push_back(str.substr(start));

    return result;
}

process::result run_command(vector<string> args, bool graceful) {
  vector<const char *> args_ptr;

  for (string &arg : args)
    args_ptr.push_back(arg.c_str());

  args_ptr.push_back(NULL);

  string command = args[0];
  string s;
  int exitcode;
  char buffer[4096 * 8];
  ssize_t nread;
  pipe_link lnk;

  if (!lnk.ok()) {
    error("Pipe failed: %s", command.c_str());
    exit(-1);
  }

  pid_t pid = fork();
  if (pid == 0) {
    dup2(DEV_NULL, STDERR_FILENO);
    dup2(lnk.write, STDOUT_FILENO);
    close(lnk.read);
    close(lnk.write);
    exit(
        execve(command.c_str(), const_cast<char *const *>(&args_ptr[0]), NULL));
  } else if (pid > 0) {
    close(lnk.write);

    while ((nread = read(lnk.read, buffer, sizeof(buffer))) > 0)
      s.append(buffer, nread);

    close(lnk.read);
    int status;
    wait(&status);

    exitcode = WEXITSTATUS(status);
    if (exitcode != 0 && !graceful) {
      error("Exec failed: %s with %i", command.c_str(), exitcode);
      exit(exitcode);
    }
    strutils::trim(s);
  } else {
    error("Fork failed: %s", command.c_str());
    exit(-1);
  }

  vector<string> lines = split_lines(s);

  return process::result{exitcode, lines};
}
} // namespace process
